@@ -27,6 +27,9 @@
 #include "sqlwchar.h"
 #include <datetime.h>
 
+bool ParamSetup(Cursor *cur, PyObject *sql, PyObject* original_params, bool skip_first);
+int BindAndConvert(Cursor *cur, Py_ssize_t i, PyObject *cell, ParamInfo *ppi);
+
 enum
 {
     CURSOR_REQUIRE_CNXN    = 0x00000001,
@@ -174,7 +177,7 @@ static bool create_name_map(Cursor* cur, SQLSMALLINT field_count, bool lower)
 
         if (!SQL_SUCCEEDED(ret))
         {
-            RaiseErrorFromHandle("SQLDescribeCol", cur->cnxn->hdbc, cur->hstmt);
+            RaiseErrorFromHandle(cur->cnxn, "SQLDescribeCol", cur->cnxn->hdbc, cur->hstmt);
             goto done;
         }
 
@@ -395,7 +398,7 @@ static void closeimpl(Cursor* cur)
 
         // If there is already an exception, don't overwrite it.
         if (!SQL_SUCCEEDED(ret) && !PyErr_Occurred())
-            RaiseErrorFromHandle("SQLFreeHandle", cur->cnxn->hdbc, SQL_NULL_HANDLE);
+            RaiseErrorFromHandle(cur->cnxn, "SQLFreeHandle", cur->cnxn->hdbc, SQL_NULL_HANDLE);
     }
 
     Py_XDECREF(cur->pPreparedSQL);
@@ -437,7 +440,7 @@ static void Cursor_dealloc(Cursor* cursor)
     {
         closeimpl(cursor);
     }
-
+    Py_XDECREF(cursor->inputsizes);
     PyObject_Del(cursor);
 }
 
@@ -483,7 +486,7 @@ bool InitColumnInfo(Cursor* cursor, SQLUSMALLINT iCol, ColumnInfo* pinfo)
     }
     if (!SQL_SUCCEEDED(ret))
     {
-        RaiseErrorFromHandle("SQLDescribeCol", cursor->cnxn->hdbc, cursor->hstmt);
+        RaiseErrorFromHandle(cursor->cnxn, "SQLDescribeCol", cursor->cnxn->hdbc, cursor->hstmt);
         return false;
     }
 
@@ -511,7 +514,7 @@ bool InitColumnInfo(Cursor* cursor, SQLUSMALLINT iCol, ColumnInfo* pinfo)
 
         if (!SQL_SUCCEEDED(ret))
         {
-            RaiseErrorFromHandle("SQLColAttribute", cursor->cnxn->hdbc, cursor->hstmt);
+            RaiseErrorFromHandle(cursor->cnxn, "SQLColAttribute", cursor->cnxn->hdbc, cursor->hstmt);
             return false;
         }
         pinfo->is_unsigned = (f == SQL_TRUE);
@@ -555,10 +558,118 @@ static bool PrepareResults(Cursor* cur, int cCols)
     return true;
 }
 
+bool ProcessDAEParams(SQLRETURN &ret, Cursor *cur, bool freeObj)
+{
+    while (ret == SQL_NEED_DATA)
+    {
+        // One or more parameters were too long to bind normally so we set the
+        // length to SQL_LEN_DATA_AT_EXEC.  ODBC will return SQL_NEED_DATA for
+        // each of the parameters we did this for.
+        //
+        // For each one we set a pointer to the ParamInfo as the "parameter
+        // data" we can access with SQLParamData.  We've stashed everything we
+        // need in there.
+
+        void* pInfo;
+        Py_BEGIN_ALLOW_THREADS
+        ret = SQLParamData(cur->hstmt, (SQLPOINTER*)&pInfo);
+        Py_END_ALLOW_THREADS
+
+        if (ret != SQL_NEED_DATA && ret != SQL_NO_DATA && !SQL_SUCCEEDED(ret))
+            return RaiseErrorFromHandle(cur->cnxn, "SQLParamData", cur->cnxn->hdbc, cur->hstmt);
+
+        TRACE("SQLParamData() --> %d\n", ret);
+        PyObject *obj = ((DAEParam*)pInfo)->cell;
+        SQLLEN maxlen = ((DAEParam*)pInfo)->maxlen;
+
+        if (ret == SQL_NEED_DATA)
+        {
+            if (PyBytes_Check(obj))
+            {
+                const char* p = PyBytes_AS_STRING(obj);
+                SQLLEN offset = 0;
+                SQLLEN cb = (SQLLEN)PyBytes_GET_SIZE(obj);
+                do
+                {
+                    SQLLEN remaining = min(maxlen, cb - offset);
+                    TRACE("SQLPutData [%d] (%d) %.10s\n", offset, remaining, &p[offset]);
+                    Py_BEGIN_ALLOW_THREADS
+                    ret = SQLPutData(cur->hstmt, (SQLPOINTER)&p[offset], remaining);
+                    Py_END_ALLOW_THREADS
+                    if (!SQL_SUCCEEDED(ret))
+                        return RaiseErrorFromHandle(cur->cnxn, "SQLPutData", cur->cnxn->hdbc, cur->hstmt);
+                    offset += remaining;
+                }
+                while (offset < cb);
+            }
+            else if (PyUnicode_Check(obj))
+            {
+                const char* p = PyUnicode_AS_DATA(obj);
+                SQLLEN offset = 0;
+                SQLLEN cb = (SQLLEN)PyUnicode_GET_SIZE(obj) * sizeof(SQLWCHAR);
+                do
+                {
+                    SQLLEN remaining = min(maxlen, cb - offset);
+                    TRACE("SQLPutData [%d] (%d) %.10s\n", offset, remaining, &p[offset]);
+                    Py_BEGIN_ALLOW_THREADS
+                    ret = SQLPutData(cur->hstmt, (SQLPOINTER)&p[offset], remaining);
+                    Py_END_ALLOW_THREADS
+                    if (!SQL_SUCCEEDED(ret))
+                        return RaiseErrorFromHandle(cur->cnxn, "SQLPutData", cur->cnxn->hdbc, cur->hstmt);
+                    offset += remaining;
+                }
+                while (offset < cb);
+            }
+#if PY_VERSION_HEX >= 0x02060000
+            else if (PyByteArray_Check(obj))
+            {
+                const char* p = PyByteArray_AS_STRING(obj);
+                SQLLEN offset = 0;
+                SQLLEN cb     = (SQLLEN)PyByteArray_GET_SIZE(obj);
+                do
+                {
+                    SQLLEN remaining = min(maxlen, cb - offset);
+                    TRACE("SQLPutData [%d] (%d) %.10s\n", offset, remaining, &p[offset]);
+                    Py_BEGIN_ALLOW_THREADS
+                    ret = SQLPutData(cur->hstmt, (SQLPOINTER)&p[offset], remaining);
+                    Py_END_ALLOW_THREADS
+                    if (!SQL_SUCCEEDED(ret))
+                        return RaiseErrorFromHandle(cur->cnxn, "SQLPutData", cur->cnxn->hdbc, cur->hstmt);
+                    offset += remaining;
+                }
+                while (offset < cb);
+            }
+#endif
+#if PY_MAJOR_VERSION < 3
+            else if (PyBuffer_Check(obj))
+            {
+                // Buffers can have multiple segments, so we might need multiple writes.  Looping through buffers isn't
+                // difficult, but we've wrapped it up in an iterator object to keep this loop simple.
+
+                BufferSegmentIterator it(obj);
+                byte* pb;
+                SQLLEN cb;
+                while (it.Next(pb, cb))
+                {
+                    Py_BEGIN_ALLOW_THREADS
+                    ret = SQLPutData(cur->hstmt, pb, cb);
+                    Py_END_ALLOW_THREADS
+                    if (!SQL_SUCCEEDED(ret))
+                        return RaiseErrorFromHandle(cur->cnxn, "SQLPutData", cur->cnxn->hdbc, cur->hstmt);
+                }
+            }
+#endif
+            if (freeObj)
+                Py_XDECREF(obj);
+            ret = SQL_NEED_DATA;
+        }
+    }
+    return true;
+}
 
 static PyObject* execute(Cursor* cur, PyObject* pSql, PyObject* params, bool skip_first)
 {
-    // Internal function to execute SQL, called by .execute and .executemany.
+    // Internal function to execute SQL, called by .execute and non-fast .executemany.
     //
     // pSql
     //   A PyString, PyUnicode, or derived object containing the SQL.
@@ -583,7 +694,6 @@ static PyObject* execute(Cursor* cur, PyObject* pSql, PyObject* params, bool ski
 
     int        params_offset = skip_first ? 1 : 0;
     Py_ssize_t cParams       = params == 0 ? 0 : PySequence_Length(params) - params_offset;
-
     SQLRETURN ret = 0;
 
     free_results(cur, FREE_STATEMENT | KEEP_PREPARED);
@@ -595,10 +705,16 @@ static PyObject* execute(Cursor* cur, PyObject* pSql, PyObject* params, bool ski
         // There are parameters, so we'll need to prepare the SQL statement and bind the parameters.  (We need to
         // prepare the statement because we can't bind a NULL (None) object without knowing the target datatype.  There
         // is no one data type that always maps to the others (no, not even varchar)).
-
-        if (!PrepareAndBind(cur, pSql, params, skip_first))
+        if (!ParamSetup(cur, pSql, params, skip_first))
             return 0;
 
+        // Detect, convert, and bind data
+        for (Py_ssize_t i = 0; i < cParams; i++)
+        {
+            Object param(PySequence_GetItem(params, i + params_offset));
+            if (!BindAndConvert(cur, i, param, &cur->paramInfos[i]))
+                return 0;
+        }
         szLastFunction = "SQLExecute";
         Py_BEGIN_ALLOW_THREADS
         ret = SQLExecute(cur->hstmt);
@@ -656,93 +772,13 @@ static PyObject* execute(Cursor* cur, PyObject* pSql, PyObject* params, bool ski
     {
         // We could try dropping through the while and if below, but if there is an error, we need to raise it before
         // FreeParameterData calls more ODBC functions.
-        RaiseErrorFromHandle("SQLExecDirectW", cur->cnxn->hdbc, cur->hstmt);
+        RaiseErrorFromHandle(cur->cnxn, "SQLExecDirectW", cur->cnxn->hdbc, cur->hstmt);
         FreeParameterData(cur);
         return 0;
     }
 
-    while (ret == SQL_NEED_DATA)
-    {
-        // One or more parameters were too long to bind normally so we set the
-        // length to SQL_LEN_DATA_AT_EXEC.  ODBC will return SQL_NEED_DATA for
-        // each of the parameters we did this for.
-        //
-        // For each one we set a pointer to the ParamInfo as the "parameter
-        // data" we can access with SQLParamData.  We've stashed everything we
-        // need in there.
-
-        szLastFunction = "SQLParamData";
-        ParamInfo* pInfo;
-        Py_BEGIN_ALLOW_THREADS
-        ret = SQLParamData(cur->hstmt, (SQLPOINTER*)&pInfo);
-        Py_END_ALLOW_THREADS
-
-        if (ret != SQL_NEED_DATA && ret != SQL_NO_DATA && !SQL_SUCCEEDED(ret))
-            return RaiseErrorFromHandle("SQLParamData", cur->cnxn->hdbc, cur->hstmt);
-
-        TRACE("SQLParamData() --> %d\n", ret);
-
-        if (ret == SQL_NEED_DATA)
-        {
-            szLastFunction = "SQLPutData";
-            if (PyBytes_Check(pInfo->pObject))
-            {
-                const char* p = PyBytes_AS_STRING(pInfo->pObject);
-                SQLLEN offset = 0;
-                SQLLEN cb = (SQLLEN)PyBytes_GET_SIZE(pInfo->pObject);
-                while (offset < cb)
-                {
-                    SQLLEN remaining = min(pInfo->maxlength, cb - offset);
-                    TRACE("SQLPutData [%d] (%d) %.10s\n", offset, remaining, &p[offset]);
-                    Py_BEGIN_ALLOW_THREADS
-                    ret = SQLPutData(cur->hstmt, (SQLPOINTER)&p[offset], remaining);
-                    Py_END_ALLOW_THREADS
-                    if (!SQL_SUCCEEDED(ret))
-                        return RaiseErrorFromHandle("SQLPutData", cur->cnxn->hdbc, cur->hstmt);
-                    offset += remaining;
-                }
-            }
-#if PY_VERSION_HEX >= 0x02060000
-            else if (PyByteArray_Check(pInfo->pObject))
-            {
-                const char* p = PyByteArray_AS_STRING(pInfo->pObject);
-                SQLLEN offset = 0;
-                SQLLEN cb     = (SQLLEN)PyByteArray_GET_SIZE(pInfo->pObject);
-                while (offset < cb)
-                {
-                    SQLLEN remaining = min(pInfo->maxlength, cb - offset);
-                    TRACE("SQLPutData [%d] (%d) %.10s\n", offset, remaining, &p[offset]);
-                    Py_BEGIN_ALLOW_THREADS
-                    ret = SQLPutData(cur->hstmt, (SQLPOINTER)&p[offset], remaining);
-                    Py_END_ALLOW_THREADS
-                    if (!SQL_SUCCEEDED(ret))
-                        return RaiseErrorFromHandle("SQLPutData", cur->cnxn->hdbc, cur->hstmt);
-                    offset += remaining;
-                }
-            }
-#endif
-#if PY_MAJOR_VERSION < 3
-            else if (PyBuffer_Check(pInfo->pObject))
-            {
-                // Buffers can have multiple segments, so we might need multiple writes.  Looping through buffers isn't
-                // difficult, but we've wrapped it up in an iterator object to keep this loop simple.
-
-                BufferSegmentIterator it(pInfo->pObject);
-                byte* pb;
-                SQLLEN cb;
-                while (it.Next(pb, cb))
-                {
-                    Py_BEGIN_ALLOW_THREADS
-                    ret = SQLPutData(cur->hstmt, pb, cb);
-                    Py_END_ALLOW_THREADS
-                    if (!SQL_SUCCEEDED(ret))
-                        return RaiseErrorFromHandle("SQLPutData", cur->cnxn->hdbc, cur->hstmt);
-                }
-            }
-#endif
-            ret = SQL_NEED_DATA;
-        }
-    }
+    if (!ProcessDAEParams(ret, cur, false))
+        return 0;
 
     FreeParameterData(cur);
 
@@ -755,14 +791,14 @@ static PyObject* execute(Cursor* cur, PyObject* pSql, PyObject* params, bool ski
     }
 
     if (!SQL_SUCCEEDED(ret))
-        return RaiseErrorFromHandle(szLastFunction, cur->cnxn->hdbc, cur->hstmt);
+        return RaiseErrorFromHandle(cur->cnxn, szLastFunction, cur->cnxn->hdbc, cur->hstmt);
 
     SQLLEN cRows = -1;
     Py_BEGIN_ALLOW_THREADS
     ret = SQLRowCount(cur->hstmt, &cRows);
     Py_END_ALLOW_THREADS
     if (!SQL_SUCCEEDED(ret))
-        return RaiseErrorFromHandle("SQLRowCount", cur->cnxn->hdbc, cur->hstmt);
+        return RaiseErrorFromHandle(cur->cnxn, "SQLRowCount", cur->cnxn->hdbc, cur->hstmt);
 
     cur->rowcount = (int)cRows;
 
@@ -777,7 +813,7 @@ static PyObject* execute(Cursor* cur, PyObject* pSql, PyObject* params, bool ski
         // Note: The SQL Server driver sometimes returns HY007 here if multiple statements (separated by ;) were
         // submitted.  This is not documented, but I've seen it with multiple successful inserts.
 
-        return RaiseErrorFromHandle("SQLNumResultCols", cur->cnxn->hdbc, cur->hstmt);
+        return RaiseErrorFromHandle(cur->cnxn, "SQLNumResultCols", cur->cnxn->hdbc, cur->hstmt);
     }
 
     TRACE("SQLNumResultCols: %d\n", cCols);
@@ -789,7 +825,7 @@ static PyObject* execute(Cursor* cur, PyObject* pSql, PyObject* params, bool ski
     }
 
     if (!SQL_SUCCEEDED(ret))
-        return RaiseErrorFromHandle("SQLRowCount", cur->cnxn->hdbc, cur->hstmt);
+        return RaiseErrorFromHandle(cur->cnxn, "SQLRowCount", cur->cnxn->hdbc, cur->hstmt);
 
     if (cCols != 0)
     {
@@ -904,18 +940,26 @@ static PyObject* Cursor_executemany(PyObject* self, PyObject* args)
             PyErr_SetString(ProgrammingError, "The second parameter to executemany must not be empty.");
             return 0;
         }
-
-        for (Py_ssize_t i = 0; i < c; i++)
+        if (cursor->fastexecmany)
         {
-            PyObject* params = PySequence_GetItem(param_seq, i);
-            PyObject* result = execute(cursor, pSql, params, false);
-            bool success = result != 0;
-            Py_XDECREF(result);
-            Py_DECREF(params);
-            if (!success)
-            {
-                cursor->rowcount = -1;
+            free_results(cursor, FREE_STATEMENT | KEEP_PREPARED);
+            if (!ExecuteMulti(cursor, pSql, param_seq))
                 return 0;
+        }
+        else
+        {
+            for (Py_ssize_t i = 0; i < c; i++)
+            {
+                PyObject* params = PySequence_GetItem(param_seq, i);
+                PyObject* result = execute(cursor, pSql, params, false);
+                bool success = result != 0;
+                Py_XDECREF(result);
+                Py_DECREF(params);
+                if (!success)
+                {
+                    cursor->rowcount = -1;
+                    return 0;
+                }
             }
         }
     }
@@ -961,6 +1005,35 @@ static PyObject* Cursor_executemany(PyObject* self, PyObject* args)
     Py_RETURN_NONE;
 }
 
+static PyObject* Cursor_setinputsizes(PyObject* self, PyObject* sizes)
+{
+    if (!Cursor_Check(self))
+    {
+        PyErr_SetString(ProgrammingError, "Invalid cursor object.");
+        return 0;
+    }
+    
+    Cursor *cur = (Cursor*)self;
+    if (Py_None == sizes)
+    {
+        Py_XDECREF(cur->inputsizes);
+        cur->inputsizes = 0;
+    }
+    else
+    {
+        if (!IsSequence(sizes))
+        {
+            PyErr_SetString(ProgrammingError, "A non-None parameter to setinputsizes must be a sequence, iterator, or generator.");
+            return 0;
+        }
+
+        Py_XDECREF(cur->inputsizes);
+        Py_INCREF(sizes);
+        cur->inputsizes = sizes;
+    }
+
+    Py_RETURN_NONE;
+}
 
 static PyObject* Cursor_fetch(Cursor* cur)
 {
@@ -988,7 +1061,7 @@ static PyObject* Cursor_fetch(Cursor* cur)
         return 0;
 
     if (!SQL_SUCCEEDED(ret))
-        return RaiseErrorFromHandle("SQLFetch", cur->cnxn->hdbc, cur->hstmt);
+        return RaiseErrorFromHandle(cur->cnxn, "SQLFetch", cur->cnxn->hdbc, cur->hstmt);
 
     field_count = PyTuple_GET_SIZE(cur->description);
 
@@ -1179,7 +1252,7 @@ static PyObject* Cursor_tables(PyObject* self, PyObject* args, PyObject* kwargs)
     const char* szTableName = 0;
     const char* szTableType = 0;
 
-    if (!PyArg_ParseTupleAndKeywords(args, kwargs, "|ssss", Cursor_tables_kwnames, &szTableName, &szCatalog, &szSchema, &szTableType))
+    if (!PyArg_ParseTupleAndKeywords(args, kwargs, "|zzzz", Cursor_tables_kwnames, &szTableName, &szCatalog, &szSchema, &szTableType))
         return 0;
 
     Cursor* cur = Cursor_Validate(self, CURSOR_REQUIRE_OPEN);
@@ -1195,14 +1268,14 @@ static PyObject* Cursor_tables(PyObject* self, PyObject* args, PyObject* kwargs)
     Py_END_ALLOW_THREADS
 
     if (!SQL_SUCCEEDED(ret))
-        return RaiseErrorFromHandle("SQLTables", cur->cnxn->hdbc, cur->hstmt);
+        return RaiseErrorFromHandle(cur->cnxn, "SQLTables", cur->cnxn->hdbc, cur->hstmt);
 
     SQLSMALLINT cCols;
     Py_BEGIN_ALLOW_THREADS
     ret = SQLNumResultCols(cur->hstmt, &cCols);
     Py_END_ALLOW_THREADS
     if (!SQL_SUCCEEDED(ret))
-        return RaiseErrorFromHandle("SQLNumResultCols", cur->cnxn->hdbc, cur->hstmt);
+        return RaiseErrorFromHandle(cur->cnxn, "SQLNumResultCols", cur->cnxn->hdbc, cur->hstmt);
 
     if (!PrepareResults(cur, cCols))
         return 0;
@@ -1248,7 +1321,7 @@ static PyObject* Cursor_columns(PyObject* self, PyObject* args, PyObject* kwargs
     const char* szTable   = 0;
     const char* szColumn  = 0;
 
-    if (!PyArg_ParseTupleAndKeywords(args, kwargs, "|ssss", Cursor_column_kwnames, &szTable, &szCatalog, &szSchema, &szColumn))
+    if (!PyArg_ParseTupleAndKeywords(args, kwargs, "|zzzz", Cursor_column_kwnames, &szTable, &szCatalog, &szSchema, &szColumn))
         return 0;
 
     Cursor* cur = Cursor_Validate(self, CURSOR_REQUIRE_OPEN);
@@ -1263,14 +1336,14 @@ static PyObject* Cursor_columns(PyObject* self, PyObject* args, PyObject* kwargs
     Py_END_ALLOW_THREADS
 
     if (!SQL_SUCCEEDED(ret))
-        return RaiseErrorFromHandle("SQLColumns", cur->cnxn->hdbc, cur->hstmt);
+        return RaiseErrorFromHandle(cur->cnxn, "SQLColumns", cur->cnxn->hdbc, cur->hstmt);
 
     SQLSMALLINT cCols;
     Py_BEGIN_ALLOW_THREADS
     ret = SQLNumResultCols(cur->hstmt, &cCols);
     Py_END_ALLOW_THREADS
     if (!SQL_SUCCEEDED(ret))
-        return RaiseErrorFromHandle("SQLNumResultCols", cur->cnxn->hdbc, cur->hstmt);
+        return RaiseErrorFromHandle(cur->cnxn, "SQLNumResultCols", cur->cnxn->hdbc, cur->hstmt);
 
     if (!PrepareResults(cur, cCols))
         return 0;
@@ -1319,7 +1392,7 @@ static PyObject* Cursor_statistics(PyObject* self, PyObject* args, PyObject* kwa
     PyObject* pUnique = Py_False;
     PyObject* pQuick  = Py_True;
 
-    if (!PyArg_ParseTupleAndKeywords(args, kwargs, "s|ssOO", Cursor_statistics_kwnames, &szTable, &szCatalog, &szSchema,
+    if (!PyArg_ParseTupleAndKeywords(args, kwargs, "s|zzOO", Cursor_statistics_kwnames, &szTable, &szCatalog, &szSchema,
                                      &pUnique, &pQuick))
         return 0;
 
@@ -1339,14 +1412,14 @@ static PyObject* Cursor_statistics(PyObject* self, PyObject* args, PyObject* kwa
     Py_END_ALLOW_THREADS
 
     if (!SQL_SUCCEEDED(ret))
-        return RaiseErrorFromHandle("SQLStatistics", cur->cnxn->hdbc, cur->hstmt);
+        return RaiseErrorFromHandle(cur->cnxn, "SQLStatistics", cur->cnxn->hdbc, cur->hstmt);
 
     SQLSMALLINT cCols;
     Py_BEGIN_ALLOW_THREADS
     ret = SQLNumResultCols(cur->hstmt, &cCols);
     Py_END_ALLOW_THREADS
     if (!SQL_SUCCEEDED(ret))
-        return RaiseErrorFromHandle("SQLNumResultCols", cur->cnxn->hdbc, cur->hstmt);
+        return RaiseErrorFromHandle(cur->cnxn, "SQLNumResultCols", cur->cnxn->hdbc, cur->hstmt);
 
     if (!PrepareResults(cur, cCols))
         return 0;
@@ -1397,7 +1470,7 @@ static PyObject* _specialColumns(PyObject* self, PyObject* args, PyObject* kwarg
     const char* szSchema  = 0;
     PyObject* pNullable = Py_True;
 
-    if (!PyArg_ParseTupleAndKeywords(args, kwargs, "s|ssO", Cursor_specialColumn_kwnames, &szTable, &szCatalog, &szSchema, &pNullable))
+    if (!PyArg_ParseTupleAndKeywords(args, kwargs, "s|zzO", Cursor_specialColumn_kwnames, &szTable, &szCatalog, &szSchema, &pNullable))
         return 0;
 
     Cursor* cur = Cursor_Validate(self, CURSOR_REQUIRE_OPEN);
@@ -1415,14 +1488,14 @@ static PyObject* _specialColumns(PyObject* self, PyObject* args, PyObject* kwarg
     Py_END_ALLOW_THREADS
 
     if (!SQL_SUCCEEDED(ret))
-        return RaiseErrorFromHandle("SQLSpecialColumns", cur->cnxn->hdbc, cur->hstmt);
+        return RaiseErrorFromHandle(cur->cnxn, "SQLSpecialColumns", cur->cnxn->hdbc, cur->hstmt);
 
     SQLSMALLINT cCols;
     Py_BEGIN_ALLOW_THREADS
     ret = SQLNumResultCols(cur->hstmt, &cCols);
     Py_END_ALLOW_THREADS
     if (!SQL_SUCCEEDED(ret))
-        return RaiseErrorFromHandle("SQLNumResultCols", cur->cnxn->hdbc, cur->hstmt);
+        return RaiseErrorFromHandle(cur->cnxn, "SQLNumResultCols", cur->cnxn->hdbc, cur->hstmt);
 
     if (!PrepareResults(cur, cCols))
         return 0;
@@ -1468,7 +1541,7 @@ static PyObject* Cursor_primaryKeys(PyObject* self, PyObject* args, PyObject* kw
     const char* szCatalog = 0;
     const char* szSchema  = 0;
 
-    if (!PyArg_ParseTupleAndKeywords(args, kwargs, "s|ss", Cursor_primaryKeys_kwnames, &szTable, &szCatalog, &szSchema))
+    if (!PyArg_ParseTupleAndKeywords(args, kwargs, "s|zz", Cursor_primaryKeys_kwnames, &szTable, &szCatalog, &szSchema))
         return 0;
 
     Cursor* cur = Cursor_Validate(self, CURSOR_REQUIRE_OPEN);
@@ -1483,14 +1556,14 @@ static PyObject* Cursor_primaryKeys(PyObject* self, PyObject* args, PyObject* kw
     Py_END_ALLOW_THREADS
 
     if (!SQL_SUCCEEDED(ret))
-        return RaiseErrorFromHandle("SQLPrimaryKeys", cur->cnxn->hdbc, cur->hstmt);
+        return RaiseErrorFromHandle(cur->cnxn, "SQLPrimaryKeys", cur->cnxn->hdbc, cur->hstmt);
 
     SQLSMALLINT cCols;
     Py_BEGIN_ALLOW_THREADS
     ret = SQLNumResultCols(cur->hstmt, &cCols);
     Py_END_ALLOW_THREADS
     if (!SQL_SUCCEEDED(ret))
-        return RaiseErrorFromHandle("SQLNumResultCols", cur->cnxn->hdbc, cur->hstmt);
+        return RaiseErrorFromHandle(cur->cnxn, "SQLNumResultCols", cur->cnxn->hdbc, cur->hstmt);
 
     if (!PrepareResults(cur, cCols))
         return 0;
@@ -1538,7 +1611,7 @@ static PyObject* Cursor_foreignKeys(PyObject* self, PyObject* args, PyObject* kw
     const char* szForeignCatalog = 0;
     const char* szForeignSchema  = 0;
 
-    if (!PyArg_ParseTupleAndKeywords(args, kwargs, "|ssssss", Cursor_foreignKeys_kwnames, &szTable, &szCatalog, &szSchema,
+    if (!PyArg_ParseTupleAndKeywords(args, kwargs, "|zzzzzz", Cursor_foreignKeys_kwnames, &szTable, &szCatalog, &szSchema,
         &szForeignTable, &szForeignCatalog, &szForeignSchema))
         return 0;
 
@@ -1555,14 +1628,14 @@ static PyObject* Cursor_foreignKeys(PyObject* self, PyObject* args, PyObject* kw
     Py_END_ALLOW_THREADS
 
     if (!SQL_SUCCEEDED(ret))
-        return RaiseErrorFromHandle("SQLForeignKeys", cur->cnxn->hdbc, cur->hstmt);
+        return RaiseErrorFromHandle(cur->cnxn, "SQLForeignKeys", cur->cnxn->hdbc, cur->hstmt);
 
     SQLSMALLINT cCols;
     Py_BEGIN_ALLOW_THREADS
     ret = SQLNumResultCols(cur->hstmt, &cCols);
     Py_END_ALLOW_THREADS
     if (!SQL_SUCCEEDED(ret))
-        return RaiseErrorFromHandle("SQLNumResultCols", cur->cnxn->hdbc, cur->hstmt);
+        return RaiseErrorFromHandle(cur->cnxn, "SQLNumResultCols", cur->cnxn->hdbc, cur->hstmt);
 
     if (!PrepareResults(cur, cCols))
         return 0;
@@ -1622,14 +1695,14 @@ static PyObject* Cursor_getTypeInfo(PyObject* self, PyObject* args, PyObject* kw
     Py_END_ALLOW_THREADS
 
     if (!SQL_SUCCEEDED(ret))
-        return RaiseErrorFromHandle("SQLGetTypeInfo", cur->cnxn->hdbc, cur->hstmt);
+        return RaiseErrorFromHandle(cur->cnxn, "SQLGetTypeInfo", cur->cnxn->hdbc, cur->hstmt);
 
     SQLSMALLINT cCols;
     Py_BEGIN_ALLOW_THREADS
     ret = SQLNumResultCols(cur->hstmt, &cCols);
     Py_END_ALLOW_THREADS
     if (!SQL_SUCCEEDED(ret))
-        return RaiseErrorFromHandle("SQLNumResultCols", cur->cnxn->hdbc, cur->hstmt);
+        return RaiseErrorFromHandle(cur->cnxn, "SQLNumResultCols", cur->cnxn->hdbc, cur->hstmt);
 
     if (!PrepareResults(cur, cCols))
         return 0;
@@ -1669,7 +1742,7 @@ static PyObject* Cursor_nextset(PyObject* self, PyObject* args)
         // Note: The SQL Server driver sometimes returns HY007 here if multiple statements (separated by ;) were
         // submitted.  This is not documented, but I've seen it with multiple successful inserts.
 
-        PyObject* pError = GetErrorFromHandle("SQLMoreResults", cur->cnxn->hdbc, cur->hstmt);
+        PyObject* pError = GetErrorFromHandle(cur->cnxn, "SQLMoreResults", cur->cnxn->hdbc, cur->hstmt);
         //
         // free_results must be run after the error has been collected
         // from the cursor as it's lost otherwise.
@@ -1704,7 +1777,7 @@ static PyObject* Cursor_nextset(PyObject* self, PyObject* args)
         // Note: The SQL Server driver sometimes returns HY007 here if multiple statements (separated by ;) were
         // submitted.  This is not documented, but I've seen it with multiple successful inserts.
 
-        PyObject* pError = GetErrorFromHandle("SQLNumResultCols", cur->cnxn->hdbc, cur->hstmt);
+        PyObject* pError = GetErrorFromHandle(cur->cnxn, "SQLNumResultCols", cur->cnxn->hdbc, cur->hstmt);
         free_results(cur, FREE_STATEMENT | KEEP_PREPARED);
         return pError;
     }
@@ -1728,7 +1801,7 @@ static PyObject* Cursor_nextset(PyObject* self, PyObject* args)
     cur->rowcount = (int)cRows;
 
     if (!SQL_SUCCEEDED(ret))
-        return RaiseErrorFromHandle("SQLRowCount", cur->cnxn->hdbc, cur->hstmt);
+        return RaiseErrorFromHandle(cur->cnxn, "SQLRowCount", cur->cnxn->hdbc, cur->hstmt);
 
     Py_RETURN_TRUE;
 }
@@ -1766,7 +1839,7 @@ static PyObject* Cursor_procedureColumns(PyObject* self, PyObject* args, PyObjec
     const char* szCatalog   = 0;
     const char* szSchema    = 0;
 
-    if (!PyArg_ParseTupleAndKeywords(args, kwargs, "|sss", Cursor_procedureColumns_kwnames, &szProcedure, &szCatalog, &szSchema))
+    if (!PyArg_ParseTupleAndKeywords(args, kwargs, "|zzz", Cursor_procedureColumns_kwnames, &szProcedure, &szCatalog, &szSchema))
         return 0;
 
     Cursor* cur = Cursor_Validate(self, CURSOR_REQUIRE_OPEN);
@@ -1782,14 +1855,14 @@ static PyObject* Cursor_procedureColumns(PyObject* self, PyObject* args, PyObjec
     Py_END_ALLOW_THREADS
 
     if (!SQL_SUCCEEDED(ret))
-        return RaiseErrorFromHandle("SQLProcedureColumns", cur->cnxn->hdbc, cur->hstmt);
+        return RaiseErrorFromHandle(cur->cnxn, "SQLProcedureColumns", cur->cnxn->hdbc, cur->hstmt);
 
     SQLSMALLINT cCols;
     Py_BEGIN_ALLOW_THREADS
     ret = SQLNumResultCols(cur->hstmt, &cCols);
     Py_END_ALLOW_THREADS
     if (!SQL_SUCCEEDED(ret))
-        return RaiseErrorFromHandle("SQLNumResultCols", cur->cnxn->hdbc, cur->hstmt);
+        return RaiseErrorFromHandle(cur->cnxn, "SQLNumResultCols", cur->cnxn->hdbc, cur->hstmt);
 
     if (!PrepareResults(cur, cCols))
         return 0;
@@ -1825,7 +1898,7 @@ static PyObject* Cursor_procedures(PyObject* self, PyObject* args, PyObject* kwa
     const char* szCatalog   = 0;
     const char* szSchema    = 0;
 
-    if (!PyArg_ParseTupleAndKeywords(args, kwargs, "|sss", Cursor_procedures_kwnames, &szProcedure, &szCatalog, &szSchema))
+    if (!PyArg_ParseTupleAndKeywords(args, kwargs, "|zzz", Cursor_procedures_kwnames, &szProcedure, &szCatalog, &szSchema))
         return 0;
 
     Cursor* cur = Cursor_Validate(self, CURSOR_REQUIRE_OPEN);
@@ -1840,14 +1913,14 @@ static PyObject* Cursor_procedures(PyObject* self, PyObject* args, PyObject* kwa
     Py_END_ALLOW_THREADS
 
     if (!SQL_SUCCEEDED(ret))
-        return RaiseErrorFromHandle("SQLProcedures", cur->cnxn->hdbc, cur->hstmt);
+        return RaiseErrorFromHandle(cur->cnxn, "SQLProcedures", cur->cnxn->hdbc, cur->hstmt);
 
     SQLSMALLINT cCols;
     Py_BEGIN_ALLOW_THREADS
     ret = SQLNumResultCols(cur->hstmt, &cCols);
     Py_END_ALLOW_THREADS
     if (!SQL_SUCCEEDED(ret))
-        return RaiseErrorFromHandle("SQLNumResultCols", cur->cnxn->hdbc, cur->hstmt);
+        return RaiseErrorFromHandle(cur->cnxn, "SQLNumResultCols", cur->cnxn->hdbc, cur->hstmt);
 
     if (!PrepareResults(cur, cCols))
         return 0;
@@ -1890,7 +1963,7 @@ static PyObject* Cursor_skip(PyObject* self, PyObject* args)
     Py_END_ALLOW_THREADS
 
     if (!SQL_SUCCEEDED(ret) && ret != SQL_NO_DATA)
-        return RaiseErrorFromHandle("SQLFetchScroll", cursor->cnxn->hdbc, cursor->hstmt);
+        return RaiseErrorFromHandle(cursor->cnxn, "SQLFetchScroll", cursor->cnxn->hdbc, cursor->hstmt);
 
     Py_RETURN_NONE;
 }
@@ -1920,6 +1993,33 @@ static PyObject* Cursor_rollback(PyObject* self, PyObject* args)
 }
 
 
+static char cancel_doc[] =
+    "Cursor.cancel() -> None\n"
+    "Cancels the processing of the current statement.\n"
+    "\n"
+    "Cancels the processing of the current statement.\n"
+    "\n"
+    "This calls SQLCancel and is designed to be called from another thread to"
+    "stop processing of an ongoing query.";
+
+static PyObject* Cursor_cancel(PyObject* self, PyObject* args)
+{
+    UNUSED(args);
+    Cursor* cur = Cursor_Validate(self, CURSOR_REQUIRE_OPEN | CURSOR_RAISE_ERROR);
+    if (!cur)
+        return 0;
+    SQLRETURN ret;
+    Py_BEGIN_ALLOW_THREADS
+    ret = SQLCancel(cur->hstmt);
+    Py_END_ALLOW_THREADS
+
+    if (!SQL_SUCCEEDED(ret))
+        return RaiseErrorFromHandle(cur->cnxn, "SQLCancel", cur->cnxn->hdbc, cur->hstmt);
+
+    Py_RETURN_NONE;
+}
+
+
 static PyObject* Cursor_ignored(PyObject* self, PyObject* args)
 {
     UNUSED(self, args);
@@ -1957,12 +2057,23 @@ static char connection_doc[] =
     "The attribute simplifies writing polymorph code in multi-connection\n" \
     "environments.";
 
+static char fastexecmany_doc[] =
+    "This read/write attribute specifies whether to use a faster executemany() which\n" \
+    "uses parameter arrays. Not all drivers may work with this implementation.";
+
+static char decimalasstring_doc[] =
+    "This read/write attribute specifies whether Decimal objects will be bound as strings\n" \
+    "if the server column is of numerical type, for drivers which do not correctly handle\n" \
+    "SQL_NUMERIC_STRUCT.";
+
 static PyMemberDef Cursor_members[] =
 {
     {"rowcount",    T_INT,       offsetof(Cursor, rowcount),        READONLY, rowcount_doc },
     {"description", T_OBJECT_EX, offsetof(Cursor, description),     READONLY, description_doc },
     {"arraysize",   T_INT,       offsetof(Cursor, arraysize),       0,        arraysize_doc },
     {"connection",  T_OBJECT_EX, offsetof(Cursor, cnxn),            READONLY, connection_doc },
+    {"fast_executemany",T_BOOL,  offsetof(Cursor, fastexecmany),    0,        fastexecmany_doc },
+    {"decimal_as_string",T_BOOL, offsetof(Cursor, decimal_as_string),0,       decimalasstring_doc },
     { 0 }
 };
 
@@ -1974,10 +2085,10 @@ static PyObject* Cursor_getnoscan(PyObject* self, void *closure)
     if (!cursor)
         return 0;
 
-    SQLUINTEGER noscan = SQL_NOSCAN_OFF;
+    SQLULEN noscan = SQL_NOSCAN_OFF;
     SQLRETURN ret;
     Py_BEGIN_ALLOW_THREADS
-    ret = SQLGetStmtAttr(cursor->hstmt, SQL_ATTR_NOSCAN, (SQLPOINTER)&noscan, sizeof(SQLUINTEGER), 0);
+    ret = SQLGetStmtAttr(cursor->hstmt, SQL_ATTR_NOSCAN, (SQLPOINTER)&noscan, sizeof(SQLULEN), 0);
     Py_END_ALLOW_THREADS
 
     if (!SQL_SUCCEEDED(ret))
@@ -2013,7 +2124,7 @@ static int Cursor_setnoscan(PyObject* self, PyObject* value, void *closure)
     Py_END_ALLOW_THREADS
     if (!SQL_SUCCEEDED(ret))
     {
-        RaiseErrorFromHandle("SQLSetStmtAttr(SQL_ATTR_NOSCAN)", cursor->cnxn->hdbc, cursor->hstmt);
+        RaiseErrorFromHandle(cursor->cnxn, "SQLSetStmtAttr(SQL_ATTR_NOSCAN)", cursor->cnxn->hdbc, cursor->hstmt);
         return -1;
     }
 
@@ -2081,6 +2192,17 @@ static char fetchmany_doc[] =
     "A ProgrammingError exception is raised if the previous call to execute() did\n" \
     "not produce any result set or no call was issued yet.";
 
+static char setinputsizes_doc[] =
+    "setinputsizes(sizes) -> None\n" \
+    "\n" \
+    "Sets the type information to be used when binding parameters.\n" \
+    "sizes must be a sequence of values, one for each input parameter.\n" \
+    "Each value may be an integer to override the column size when binding character\n" \
+    "data, a Type Object to override the SQL type, or a sequence of integers to specify\n" \
+    "(SQL type, column size, decimal digits) where any may be none to use the default.\n" \
+    "\n" \
+    "Parameters beyond the length of the sequence will be bound with the defaults.\n" \
+    "Setting sizes to None reverts all parameters to the defaults.";
 
 static char enter_doc[] = "__enter__() -> self.";
 static PyObject* Cursor_enter(PyObject* self, PyObject* args)
@@ -2108,7 +2230,7 @@ static PyObject* Cursor_exit(PyObject* self, PyObject* args)
         Py_END_ALLOW_THREADS
 
         if (!SQL_SUCCEEDED(ret))
-            return RaiseErrorFromHandle("SQLEndTran(SQL_COMMIT)", cursor->cnxn->hdbc, cursor->hstmt);
+            return RaiseErrorFromHandle(cursor->cnxn, "SQLEndTran(SQL_COMMIT)", cursor->cnxn->hdbc, cursor->hstmt);
     }
 
     Py_RETURN_NONE;
@@ -2120,7 +2242,7 @@ static PyMethodDef Cursor_methods[] =
     { "close",            (PyCFunction)Cursor_close,            METH_NOARGS,                close_doc            },
     { "execute",          (PyCFunction)Cursor_execute,          METH_VARARGS,               execute_doc          },
     { "executemany",      (PyCFunction)Cursor_executemany,      METH_VARARGS,               executemany_doc      },
-    { "setinputsizes",    (PyCFunction)Cursor_ignored,          METH_VARARGS,               ignored_doc          },
+    { "setinputsizes",    (PyCFunction)Cursor_setinputsizes,    METH_O,                     setinputsizes_doc    },
     { "setoutputsize",    (PyCFunction)Cursor_ignored,          METH_VARARGS,               ignored_doc          },
     { "fetchval",         (PyCFunction)Cursor_fetchval,         METH_NOARGS,                fetchval_doc         },
     { "fetchone",         (PyCFunction)Cursor_fetchone,         METH_NOARGS,                fetchone_doc         },
@@ -2140,9 +2262,10 @@ static PyMethodDef Cursor_methods[] =
     { "skip",             (PyCFunction)Cursor_skip,             METH_VARARGS,               skip_doc             },
     { "commit",           (PyCFunction)Cursor_commit,           METH_NOARGS,                commit_doc           },
     { "rollback",         (PyCFunction)Cursor_rollback,         METH_NOARGS,                rollback_doc         },
-    { "__enter__",        Cursor_enter,                         METH_NOARGS,                enter_doc            },
-    { "__exit__",         Cursor_exit,                          METH_VARARGS,               exit_doc             },
-    { 0, 0, 0, 0 }
+    {"cancel",           (PyCFunction)Cursor_cancel,           METH_NOARGS,                cancel_doc},
+    {"__enter__",        Cursor_enter,                         METH_NOARGS,                enter_doc            },
+    {"__exit__",         Cursor_exit,                          METH_VARARGS,               exit_doc             },
+    {0, 0, 0, 0}
 };
 
 static char cursor_doc[] =
@@ -2233,10 +2356,13 @@ Cursor_New(Connection* cnxn)
         cur->paramcount        = 0;
         cur->paramtypes        = 0;
         cur->paramInfos        = 0;
+        cur->inputsizes        = 0;
         cur->colinfos          = 0;
         cur->arraysize         = 1;
         cur->rowcount          = -1;
         cur->map_name_to_index = 0;
+        cur->fastexecmany      = 0;
+        cur->decimal_as_string = 0;
 
         Py_INCREF(cnxn);
         Py_INCREF(cur->description);
@@ -2248,7 +2374,7 @@ Cursor_New(Connection* cnxn)
 
         if (!SQL_SUCCEEDED(ret))
         {
-            RaiseErrorFromHandle("SQLAllocHandle", cnxn->hdbc, SQL_NULL_HANDLE);
+            RaiseErrorFromHandle(cnxn, "SQLAllocHandle", cnxn->hdbc, SQL_NULL_HANDLE);
             Py_DECREF(cur);
             return 0;
         }
@@ -2261,7 +2387,7 @@ Cursor_New(Connection* cnxn)
 
             if (!SQL_SUCCEEDED(ret))
             {
-                RaiseErrorFromHandle("SQLSetStmtAttr(SQL_ATTR_QUERY_TIMEOUT)", cnxn->hdbc, cur->hstmt);
+                RaiseErrorFromHandle(cnxn, "SQLSetStmtAttr(SQL_ATTR_QUERY_TIMEOUT)", cnxn->hdbc, cur->hstmt);
                 Py_DECREF(cur);
                 return 0;
             }
