@@ -1,10 +1,9 @@
-
 /*
  * Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated
  * documentation files (the "Software"), to deal in the Software without restriction, including without limitation the
  * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to
  * permit persons to whom the Software is furnished to do so.
- * 
+ *
  * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
  * WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS
  * OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR
@@ -16,6 +15,12 @@
 
 struct Connection;
 
+struct DAEParam
+{
+    PyObject *cell;
+    SQLLEN maxlen;
+};
+
 struct ColumnInfo
 {
     SQLSMALLINT sql_type;
@@ -45,7 +50,6 @@ struct ParamInfo
     // The value pointer that will be bound.  If `alloc` is true, this was allocated with malloc and must be freed.
     // Otherwise it is zero or points into memory owned by the original Python parameter.
     SQLPOINTER ParameterValuePtr;
-
     SQLLEN BufferLength;
     SQLLEN StrLen_or_Ind;
 
@@ -75,7 +79,11 @@ struct ParamInfo
         TIMESTAMP_STRUCT timestamp;
         DATE_STRUCT date;
         TIME_STRUCT time;
+        DAEParam dae;
     } Data;
+    // To ensure enough space for the indicator, which follows the data immediately.
+    // This field is not written directly but is used to occupy its space.
+    SQLLEN dummyStrLen_or_Ind;
 };
 
 struct Cursor
@@ -91,7 +99,7 @@ struct Cursor
     //
     // SQL Parameters
     //
-    
+
     // If non-zero, a pointer to the previously prepared SQL string, allowing us to skip the prepare and gathering of
     // parameter data.
     PyObject* pPreparedSQL;
@@ -112,6 +120,18 @@ struct Cursor
     // bind into the Python objects directly.
     ParamInfo* paramInfos;
 
+    // Parameter set array (used with executemany)
+    unsigned char *paramArray;
+    
+    // Whether to use fast executemany with parameter arrays and other optimisations
+    char fastexecmany;
+    
+    // Whether to bind DECIMAL as string, for those drivers that can't handle SQL_NUMERIC_STRUCT
+    char decimal_as_string;
+    
+    // The list of information for setinputsizes().
+    PyObject *inputsizes;
+
     //
     // Result Information
     //
@@ -144,5 +164,6 @@ void Cursor_init();
 
 Cursor* Cursor_New(Connection* cnxn);
 PyObject* Cursor_execute(PyObject* self, PyObject* args);
+bool ProcessDAEParams(SQLRETURN &ret, Cursor *cur, bool freeObj);
 
 #endif
