@@ -1,7 +1,11 @@
 
 #include "pyodbc.h"
+#include "wrapper.h"
+#include "textenc.h"
+#include "connection.h"
 #include "errors.h"
 #include "pyodbcmodule.h"
+#include "sqlwchar.h"
 
 // Exceptions
 
@@ -15,15 +19,27 @@ struct SqlStateMapping
 
 static const struct SqlStateMapping sql_state_mapping[] =
 {
+    { "01002", 5, &OperationalError },
+    { "08001", 5, &OperationalError },
+    { "08003", 5, &OperationalError },
+    { "08004", 5, &OperationalError },
+    { "08007", 5, &OperationalError },
+    { "08S01", 5, &OperationalError },
     { "0A000", 5, &NotSupportedError },
+    { "28000", 5, &InterfaceError },
     { "40002", 5, &IntegrityError },
     { "22",    2, &DataError },
     { "23",    2, &IntegrityError },
     { "24",    2, &ProgrammingError },
     { "25",    2, &ProgrammingError },
     { "42",    2, &ProgrammingError },
+    { "HY001", 5, &OperationalError },
+    { "HY014", 5, &OperationalError },
     { "HYT00", 5, &OperationalError },
     { "HYT01", 5, &OperationalError },
+    { "IM001", 5, &InterfaceError },
+    { "IM002", 5, &InterfaceError },
+    { "IM003", 5, &InterfaceError },
 };
 
 
@@ -163,11 +179,11 @@ static PyObject* GetError(const char* sqlstate, PyObject* exc_class, PyObject* p
 
 static const char* DEFAULT_ERROR = "The driver did not supply an error!";
 
-PyObject* RaiseErrorFromHandle(const char* szFunction, HDBC hdbc, HSTMT hstmt)
+PyObject* RaiseErrorFromHandle(Connection *conn, const char* szFunction, HDBC hdbc, HSTMT hstmt)
 {
     // The exception is "set" in the interpreter.  This function returns 0 so this can be used in a return statement.
 
-    PyObject* pError = GetErrorFromHandle(szFunction, hdbc, hstmt);
+    PyObject* pError = GetErrorFromHandle(conn, szFunction, hdbc, hstmt);
 
     if (pError)
     {
@@ -179,7 +195,37 @@ PyObject* RaiseErrorFromHandle(const char* szFunction, HDBC hdbc, HSTMT hstmt)
 }
 
 
-PyObject* GetErrorFromHandle(const char* szFunction, HDBC hdbc, HSTMT hstmt)
+inline void CopySqlState(const ODBCCHAR* src, char* dest)
+{
+    // Copies a SQLSTATE read as SQLWCHAR into a character buffer.  We know that SQLSTATEs are
+    // composed of ASCII characters and we need one standard to compare when choosing
+    // exceptions.
+    //
+    // Strangely, even when the error messages are UTF-8, PostgreSQL and MySQL encode the
+    // sqlstate as UTF-16LE.  We'll simply copy all non-zero bytes, with some checks for
+    // running off the end of the buffers which will work for ASCII, UTF8, and UTF16 LE & BE.
+    // It would work for UTF32 if I increase the size of the ODBCCHAR buffer to handle it.
+    //
+    // (In the worst case, if a driver does something totally weird, we'll have an incomplete
+    // SQLSTATE.)
+    //
+
+    const char* pchSrc = (const char*)src;
+    const char* pchSrcMax = pchSrc + sizeof(ODBCCHAR) * 5;
+    char* pchDest = dest;         // Where we are copying into dest
+    char* pchDestMax = dest + 5;  // We know a SQLSTATE is 5 characters long
+
+    while (pchDest < pchDestMax && pchSrc < pchSrcMax)
+    {
+        if (*pchSrc)
+            *pchDest++ = *pchSrc;
+        pchSrc++;
+    }
+    *pchDest = 0;
+}
+
+
+PyObject* GetErrorFromHandle(Connection *conn, const char* szFunction, HDBC hdbc, HSTMT hstmt)
 {
     TRACE("In RaiseError(%s)!\n", szFunction);
 
@@ -201,11 +247,8 @@ PyObject* GetErrorFromHandle(const char* szFunction, HDBC hdbc, HSTMT hstmt)
     SQLINTEGER nNativeError;
     SQLSMALLINT cchMsg;
 
-    char sqlstateT[6];
-    char szMsg[1024];
-
-    PyObject* pMsg = 0;
-    PyObject* pMsgPart = 0;
+    ODBCCHAR sqlstateT[6];
+    ODBCCHAR szMsg[1024];
 
     if (hstmt != SQL_NULL_HANDLE)
     {
@@ -228,6 +271,8 @@ PyObject* GetErrorFromHandle(const char* szFunction, HDBC hdbc, HSTMT hstmt)
 
     SQLSMALLINT iRecord = 1;
 
+    Object msg;
+
     for (;;)
     {
         szMsg[0]     = 0;
@@ -237,7 +282,7 @@ PyObject* GetErrorFromHandle(const char* szFunction, HDBC hdbc, HSTMT hstmt)
 
         SQLRETURN ret;
         Py_BEGIN_ALLOW_THREADS
-        ret = SQLGetDiagRec(nHandleType, h, iRecord, (SQLCHAR*)sqlstateT, &nNativeError, (SQLCHAR*)szMsg, (short)(_countof(szMsg)-1), &cchMsg);
+        ret = SQLGetDiagRecW(nHandleType, h, iRecord, (SQLWCHAR*)sqlstateT, &nNativeError, (SQLWCHAR*)szMsg, (short)(_countof(szMsg)-1), &cchMsg);
         Py_END_ALLOW_THREADS
         if (!SQL_SUCCEEDED(ret))
             break;
@@ -245,29 +290,34 @@ PyObject* GetErrorFromHandle(const char* szFunction, HDBC hdbc, HSTMT hstmt)
         // Not always NULL terminated (MS Access)
         sqlstateT[5] = 0;
 
-        if (cchMsg != 0)
+        // For now, default to UTF-16LE if this is not in the context of a connection.
+        // Note that this will not work if the DM is using a different wide encoding (e.g. UTF-32).
+        const char *unicode_enc = conn ? conn->metadata_enc.name : "utf-16-le";
+        Object msgStr(PyUnicode_Decode((char*)szMsg, cchMsg * sizeof(ODBCCHAR), unicode_enc, "strict"));
+
+        if (cchMsg != 0 && msgStr.Get())
         {
             if (iRecord == 1)
             {
-                // This is the first error message, so save the SQLSTATE for determining the exception class and append
-                // the calling function name.
-
-                memcpy(sqlstate, sqlstateT, sizeof(sqlstate[0]) * _countof(sqlstate));
-
-                pMsg = PyString_FromFormat("[%s] %s (%ld) (%s)", sqlstateT, szMsg, (long)nNativeError, szFunction);
-                if (pMsg == 0)
+                // This is the first error message, so save the SQLSTATE for determining the
+                // exception class and append the calling function name.
+                CopySqlState(sqlstateT, sqlstate);
+                msg = PyUnicode_FromFormat("[%s] %V (%ld) (%s)", sqlstate, msgStr.Get(), "(null)", (long)nNativeError, szFunction);
+                if (!msg)
                     return 0;
             }
             else
             {
                 // This is not the first error message, so append to the existing one.
-                pMsgPart = PyString_FromFormat("; [%s] %s (%ld)", sqlstateT, szMsg, (long)nNativeError);
-                if (pMsgPart == 0)
-                {
-                    Py_XDECREF(pMsg);
-                    return 0;
-                }
-                PyString_ConcatAndDel(&pMsg, pMsgPart);
+                Object more(PyUnicode_FromFormat("; [%s] %V (%ld)", sqlstate, msgStr.Get(), "(null)", (long)nNativeError));
+                if (!more)
+                    break;  // Something went wrong, but we'll return the msg we have so far
+
+                Object both(PyUnicode_Concat(msg, more));
+                if (!both)
+                    break;
+
+                msg = both.Detach();
             }
         }
 
@@ -279,20 +329,20 @@ PyObject* GetErrorFromHandle(const char* szFunction, HDBC hdbc, HSTMT hstmt)
 #endif
     }
 
-    if (pMsg == 0)
+    if (!msg || PyUnicode_GetSize(msg.Get()) == 0)
     {
         // This only happens using unixODBC.  (Haven't tried iODBC yet.)  Either the driver or the driver manager is
         // buggy and has signaled a fault without recording error information.
         sqlstate[0] = '\0';
-        pMsg = PyString_FromString(DEFAULT_ERROR);
-        if (pMsg == 0)
+        msg = PyString_FromString(DEFAULT_ERROR);
+        if (!msg)
         {
             PyErr_NoMemory();
             return 0;
         }
     }
 
-    return GetError(sqlstate, 0, pMsg);
+    return GetError(sqlstate, 0, msg.Detach());
 }
 
 
