@@ -592,11 +592,10 @@ class MySqlTestCase(unittest.TestCase):
         row = self.cursor.execute("select * from t1").fetchone()
         self.assertEquals(self.cursor.description, row.cursor_description)
 
-
     def test_executemany(self):
         self.cursor.execute("create table t1(a int, b varchar(10))")
 
-        params = [ (i, str(i)) for i in range(1, 6) ]
+        params = [(i, str(i)) for i in range(1, 6)]
 
         self.cursor.executemany("insert into t1(a, b) values (?,?)", params)
 
@@ -611,6 +610,26 @@ class MySqlTestCase(unittest.TestCase):
             self.assertEqual(param[0], row[0])
             self.assertEqual(param[1], row[1])
 
+    def test_fast_executemany(self):
+
+        self.cursor.fast_executemany = True
+
+        self.cursor.execute("create table t1(a int, b varchar(10))")
+
+        params = [(i, str(i)) for i in range(1, 6)]
+
+        self.cursor.executemany("insert into t1(a, b) values (?,?)", params)
+
+        count = self.cursor.execute("select count(*) from t1").fetchone()[0]
+        self.assertEqual(count, len(params))
+
+        self.cursor.execute("select a, b from t1 order by a")
+        rows = self.cursor.fetchall()
+        self.assertEqual(count, len(rows))
+
+        for param, row in zip(params, rows):
+            self.assertEqual(param[0], row[0])
+            self.assertEqual(param[1], row[1])
 
     def test_executemany_one(self):
         "Pass executemany a single sequence"
